/*******************************************************************************
* File Name: PGA_follower_PM.c
* Version 1.80
*
* Description:
*  This file provides the power management source code to the API for the 
*  OpAmp (Analog Buffer) component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "PGA_follower.h"

static PGA_follower_BACKUP_STRUCT  PGA_follower_backup;


/*******************************************************************************  
* Function Name: PGA_follower_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration registers.
* 
* Parameters:
*  void
* 
* Return:
*  void
*
*******************************************************************************/
void PGA_follower_SaveConfig(void) 
{
    /* Nothing to save as registers are System reset on retention flops */
}


/*******************************************************************************  
* Function Name: PGA_follower_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration registers.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void PGA_follower_RestoreConfig(void) 
{
    /* Nothing to restore */
}


/*******************************************************************************   
* Function Name: PGA_follower_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves its configuration. Should be called 
*  just prior to entering sleep.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  PGA_follower_backup: The structure field 'enableState' is modified 
*  depending on the enable state of the block before entering to sleep mode.
*
*******************************************************************************/
void PGA_follower_Sleep(void) 
{
    /* Save OpAmp enable state */
    if(PGA_follower_ACT_PWR_EN == (PGA_follower_PM_ACT_CFG_REG & PGA_follower_ACT_PWR_EN))
    {
        /* Component is enabled */
        PGA_follower_backup.enableState = 1u;
    }
    else
    {
        /* Component is disabled */
        PGA_follower_backup.enableState = 0u;
    }
    /* Stops the component */
    PGA_follower_Stop();

    /* Saves the configuration */
    PGA_follower_SaveConfig();
}


/*******************************************************************************  
* Function Name: PGA_follower_Wakeup
********************************************************************************
*
* Summary:
*  Enables block's operation and restores its configuration. Should be called
*  just after awaking from sleep.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  PGA_follower_backup: The structure field 'enableState' is used to 
*  restore the enable state of block after wakeup from sleep mode.
*
*******************************************************************************/
void PGA_follower_Wakeup(void) 
{
    /* Restore the user configuration */
    PGA_follower_RestoreConfig();

    /* Enables the component operation */
    if(PGA_follower_backup.enableState == 1u)
    {
        PGA_follower_Enable();
    } /* Do nothing if component was disable before */
}


/* [] END OF FILE */
